//
//  MailboxInteractor.swift
//  ProtonMailMac
//
//  Created by Marcel Piešťanský on 16.09.2021.
//  Copyright (c) 2021 marpies. All rights reserved.
//

import Foundation

protocol MailboxBusinessLogic {
    func loadItems(request: Mailbox.LoadItems.Request)
    func loadPage(request: Mailbox.LoadPage.Request)
    func processErrorViewButtonTap()
    func updateItemStar(request: Mailbox.UpdateItemStar.Request)
    func processItemsSelection(request: Mailbox.ItemsDidSelect.Request)
    func processItemsDeselection()
}

protocol MailboxDataStore {
    
}

class MailboxInteractor: MailboxBusinessLogic, MailboxDataStore, MailboxWorkerDelegate {
    
    var worker: MailboxWorker?
    
    var presenter: MailboxPresentationLogic?
    
    //
    // MARK: - Load items
    //
    
    func loadItems(request: Mailbox.LoadItems.Request) {
        self.worker?.delegate = self
        self.worker?.loadItems(request: request)
    }
    
    //
    // MARK: - Load page
    //
    
    func loadPage(request: Mailbox.LoadPage.Request) {
        self.worker?.loadPage(request: request)
    }
    
    //
    // MARK: - Process error view button tap
    //
    
    func processErrorViewButtonTap() {
        self.worker?.refreshMailbox(eventsOnly: true)
    }
    
    //
    // MARK: - Star / unstar item
    //
    
    func updateItemStar(request: Mailbox.UpdateItemStar.Request) {
        self.worker?.updateItemStar(request: request)
    }
    
    //
    // MARK: - Process items selection
    //
    
    func processItemsSelection(request: Mailbox.ItemsDidSelect.Request) {
        self.worker?.processItemsSelection(request: request)
    }
    
    //
    // MARK: - Process items deselection
    //
    
    func processItemsDeselection() {
        self.worker?.processItemsDeselection()
    }
    
    //
    // MARK: - Worker delegate
    //
    
    func conversationsDidLoad(response: Conversations.LoadConversations.Response) {
        self.presenter?.presentConversations(response: response)
    }
    
    func conversationsDidUpdate(response: Conversations.UpdateConversations.Response) {
        self.presenter?.presentConversationsUpdate(response: response)
    }
    
    func conversationDidUpdate(response: Conversations.UpdateConversation.Response) {
        self.presenter?.presentConversationUpdate(response: response)
    }
    
    func conversationsDidRefresh(response: Conversations.RefreshConversations.Response) {
        self.presenter?.presentConversationsRefresh(response: response)
    }
    
    func loadDidFail(response: Mailbox.LoadError.Response) {
        self.presenter?.presentLoadError(response: response)
    }
    
    func conversationShouldLoad(response: Mailbox.LoadConversation.Response) {
        self.presenter?.presentLoadConversation(response: response)
    }
    
    func messagesDidLoad(response: Messages.LoadMessages.Response) {
        self.presenter?.presentMessages(response: response)
    }
    
    func messageDidUpdate(response: Messages.UpdateMessage.Response) {
        self.presenter?.presentMessageUpdate(response: response)
    }
    
    func messagesDidRefresh(response: Messages.RefreshMessages.Response) {
        self.presenter?.presentMessagesRefresh(response: response)
    }
    
    func messagesDidUpdate(response: Messages.UpdateMessages.Response) {
        self.presenter?.presentMessagesUpdate(response: response)
    }
    
    func mailboxDidUpdateWithoutChange() {
        self.presenter?.presentItemsUpToDate()
    }
    
    func mailboxSelectionDidUpdate(response: Mailbox.ItemsDidSelect.Response) {
        self.presenter?.presentItemsSelection(response: response)
    }
    
    func mailboxPageCountDidUpdate(response: Mailbox.PageCountUpdate.Response) {
        self.presenter?.presentPageCountUpdate(response: response)
    }
    
}
